# -*- coding: utf-8 -*-

import unittest
import logging

from pyutil_cfg import cfg


class TestCfg(unittest.TestCase):

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_init(self):
        logger, config = cfg.init('test_cfg', 'tests/data/test.ini', params={'test_params': 'test'})

        assert config['a'] == 'b'
        assert config['test_params'] == 'test'
        assert config['test_ini'] == 'test2'
        assert config['test_list'] == ['test0', 'test1', 2]
        assert config['test_dict'] == {'test_key': 'test_val', 'test_key2': 'test_val2'}
        assert config['test_set'] == set(['a', 'b', 'c'])
        assert config['test_list_dict'] == [
            {'A': 1, 'B': 'b'},
            {'A': 2, 'B': 'c'},
            {'A': 3, 'B': 'd'},
            {'A': 4, 'B': 'e'},
        ]

    def test_init_no_logger(self):
        logger, config = cfg.init('test_cfg', 'tests/data/test_no_logger.ini', params={'test_params': 'test'})

        assert config['a'] == 'b'
        assert config['test_params'] == 'test'
        assert config['test_ini'] == 'test2'
        assert config['test_list'] == ['test0', 'test1', 2]
        assert config['test_dict'] == {'test_key': 'test_val', 'test_key2': 'test_val2'}
        assert config['test_set'] == set(["a", "b", "c"])

    def test_init_none(self):
        logger, config = cfg.init('test_cfg', 'tests/data/test_none.ini', params={'test_params': 'test'})

        assert type(config) == dict
        assert config['test_params'] == 'test'



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyutil-cfg",
    version='0.0.6',
    author="chhsiao",
    author_email="hsiao.chuanheng@gmail.com",
    description="python util for cfg (configurations)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/chhsiao1981/pyutil_cfg",
    packages=setuptools.find_packages(exclude=["tests"]),
    install_requires=[
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



# -*- coding: utf-8 -*-

from configparser import SafeConfigParser
import logging
import logging.config

import json


def init(name, ini_filename, log_ini_filename='', params=None):
    logger = _init_logger(name, log_ini_filename, ini_filename)
    config = _init_ini_file(name, ini_filename, logger)
    config = _post_init_config(params, config, logger)

    logger.debug('pyutil_cfg: to return: name: %s logger: %s config: %s', name, logger, config)

    return logger, config


def _init_logger(name, log_ini_filename, ini_filename):
    logger = logging.getLogger(name)

    if not log_ini_filename:
        log_ini_filename = ini_filename

    if not log_ini_filename:
        return logger

    try:
        logging.config.fileConfig(log_ini_filename, disable_existing_loggers=False)
    except:
        pass

    return logger


def _init_ini_file(name, ini_filename, logger):
    '''
    setup name:main config
    '''
    section = name + ':main'

    config = _init_ini_file_core(ini_filename, section, logger)

    return config


def _init_ini_file_core(ini_filename, section, logger):
    '''
    get ini conf from section
    return: config: {key: val} val: json_loaded
    '''
    if not ini_filename:
        return {}

    config_parser = SafeConfigParser()
    config_parser.read(ini_filename)
    sections = config_parser.sections()
    if section not in sections:
        return {}

    options = config_parser.options(section)
    config = {option: _init_ini_file_parse_option(option, section, config_parser, logger) for option in options}

    return config


def _init_ini_file_parse_option(option, section, config_parser, logger):
    try:
        val = config_parser.get(section, option)
    except Exception as e:
        logger.exception('unable to get option: section: %s option: %s e: %s', section, option, e)
        val = ''
    return _init_ini_file_val_to_json(option, val)


def _init_ini_file_val_to_json(option, val):
    '''
    try to do json load on value
    '''

    if val.__class__.__name__ != 'str':
        return val

    orig_v = val
    try:
        val = json.loads(val)
    except:
        val = orig_v

    if option.endswith('_set'):
        val = set(val)

    return val


def _post_init_config(params, config, logger):
    '''
    add additional parameters into config
    '''

    if not params:
        return config

    for (k, v) in params.items():
        if k in config:
            logger.warning('config will be overwritten by params: key: %s origin: %s new: %s', k, config[k], v)

    config.update(params)

    return config
